@@ -1,8 +1,9 @@
 from sympy.concrete import Sum
-from sympy.concrete.delta import deltaproduct as dp, deltasummation as ds
+from sympy.concrete.delta import deltaproduct as dp, deltasummation as ds, _extract_delta
 from sympy.core import Eq, S, symbols, oo
 from sympy.functions import KroneckerDelta as KD, Piecewise, piecewise_fold
 from sympy.logic import And
+from sympy.utilities.pytest import raises
 
 i, j, k, l, m = symbols("i j k l m", integer=True, finite=True)
 x, y = symbols("x y", commutative=False)
@@ -501,3 +502,7 @@ def test_deltasummation_mul_add_x_kd_add_y_kd():
     assert ds((x + KD(i, k))*(y + KD(i, j)), (j, k, l)) == piecewise_fold(
         Piecewise((KD(i, k) + x, And(k <= i, i <= l)), (0, True)) +
         (l - k + 1)*(KD(i, k) + x)*y)
+
+
+def test_extract_delta():
+    raises(ValueError, lambda: _extract_delta(KD(i, j) + KD(k, l), i))
