@@ -1,11 +1,11 @@
 from __future__ import print_function, division
 
-from sympy.core.function import Function
 from sympy.core import S, Integer
-from sympy.core.mul import prod
+from sympy.core.compatibility import range, SYMPY_INTS
+from sympy.core.function import Function
 from sympy.core.logic import fuzzy_not
+from sympy.core.mul import prod
 from sympy.utilities.iterables import (has_dups, default_sort_key)
-from sympy.core.compatibility import range
 
 ###############################################################################
 ###################### Kronecker Delta, Levi-Civita etc. ######################
@@ -72,7 +72,7 @@ class LeviCivita(Function):
 
     @classmethod
     def eval(cls, *args):
-        if all(isinstance(a, (int, Integer)) for a in args):
+        if all(isinstance(a, (SYMPY_INTS, Integer)) for a in args):
             return eval_levicivita(*args)
         if has_dups(args):
             return S.Zero
@@ -127,7 +127,7 @@ class KroneckerDelta(Function):
     References
     ==========
 
-    .. [1] http://en.wikipedia.org/wiki/Kronecker_delta
+    .. [1] https://en.wikipedia.org/wiki/Kronecker_delta
     """
 
     is_integer = True
@@ -437,10 +437,6 @@ class KroneckerDelta(Function):
         else:
             return 0
 
-    @staticmethod
-    def _latex_no_arg(printer):
-        return r'\delta'
-
     @property
     def indices(self):
         return self.args[0:2]
