@@ -6,21 +6,30 @@ from __future__ import print_function, division
 import random
 import math
 
-from .primetest import isprime
-from .generate import sieve, primerange, nextprime
 from sympy.core import sympify
+from sympy.core.compatibility import as_int, SYMPY_INTS, range, string_types
 from sympy.core.evalf import bitcount
+from sympy.core.expr import Expr
+from sympy.core.function import Function
 from sympy.core.logic import fuzzy_and
+from sympy.core.mul import Mul
 from sympy.core.numbers import igcd, ilcm, Rational
 from sympy.core.power import integer_nthroot, Pow
-from sympy.core.mul import Mul
-from sympy.core.compatibility import as_int, SYMPY_INTS, range
 from sympy.core.singleton import S
-from sympy.core.function import Function
-from sympy.core.expr import Expr
+from .primetest import isprime
+from .generate import sieve, primerange, nextprime
 
-small_trailing = [i and max(int(not i % 2**j) and j for j in range(1, 8))
-    for i in range(256)]
+
+# Note: This list should be updated whenever new Mersenne primes are found.
+# Refer: https://www.mersenne.org/
+MERSENNE_PRIME_EXPONENTS = (2, 3, 5, 7, 13, 17, 19, 31, 61, 89, 107, 127, 521, 607, 1279, 2203,
+ 2281, 3217, 4253, 4423, 9689, 9941, 11213, 19937, 21701, 23209, 44497, 86243, 110503, 132049,
+ 216091, 756839, 859433, 1257787, 1398269, 2976221, 3021377, 6972593, 13466917, 20996011, 24036583,
+ 25964951, 30402457, 32582657, 37156667, 42643801, 43112609, 57885161, 74207281, 77232917, 82589933)
+
+small_trailing = [0] * 256
+for j in range(1,8):
+    small_trailing[1<<j::1<<(j+1)] = [j] * (1<<(7-j))
 
 
 def smoothness(n):
@@ -30,6 +39,9 @@ def smoothness(n):
     The smoothness of n is the largest prime factor of n; the power-
     smoothness is the largest divisor raised to its multiplicity.
 
+    Examples
+    ========
+
     >>> from sympy.ntheory.factor_ import smoothness
     >>> smoothness(2**7*3**2)
     (3, 128)
@@ -117,7 +129,7 @@ def smoothness_p(n, m=-1, power=0, visual=None):
     elif visual not in (True, False):
         visual = None
 
-    if type(n) is str:
+    if isinstance(n, string_types):
         if visual:
             return n
         d = {}
@@ -180,6 +192,17 @@ def trailing(n):
         if n == 1 << z:
             return z
 
+    if z < 300:
+        # fixed 8-byte reduction
+        t = 8
+        n >>= 8
+        while not n & 0xff:
+            n >>= 8
+            t += 8
+        return t + small_trailing[n & 0xff]
+
+    # binary reduction important when there might be a large
+    # number of trailing 0s
     t = 0
     p = 8
     while not n & 1:
@@ -210,25 +233,22 @@ def multiplicity(p, n):
         p, n = as_int(p), as_int(n)
     except ValueError:
         if all(isinstance(i, (SYMPY_INTS, Rational)) for i in (p, n)):
-            try:
-                p = Rational(p)
-                n = Rational(n)
-                if p.q == 1:
-                    if n.p == 1:
-                        return -multiplicity(p.p, n.q)
-                    return S.Zero
-                elif p.p == 1:
-                    return multiplicity(p.q, n.q)
-                else:
-                    like = min(
-                        multiplicity(p.p, n.p),
-                        multiplicity(p.q, n.q))
-                    cross = min(
-                        multiplicity(p.q, n.p),
-                        multiplicity(p.p, n.q))
-                    return like - cross
-            except AttributeError:
-                pass
+            p = Rational(p)
+            n = Rational(n)
+            if p.q == 1:
+                if n.p == 1:
+                    return -multiplicity(p.p, n.q)
+                return multiplicity(p.p, n.p) - multiplicity(p.p, n.q)
+            elif p.p == 1:
+                return multiplicity(p.q, n.q)
+            else:
+                like = min(
+                    multiplicity(p.p, n.p),
+                    multiplicity(p.q, n.q))
+                cross = min(
+                    multiplicity(p.q, n.p),
+                    multiplicity(p.p, n.q))
+                return like - cross
         raise ValueError('expecting ints or fractions, got %s and %s' % (p, n))
 
     if n == 0:
@@ -449,8 +469,8 @@ def pollard_rho(n, s=2, a=1, retries=5, seed=1234, max_steps=None, F=None):
     References
     ==========
 
-    - Richard Crandall & Carl Pomerance (2005), "Prime Numbers:
-      A Computational Perspective", Springer, 2nd edition, 229-231
+    .. [1] Richard Crandall & Carl Pomerance (2005), "Prime Numbers:
+           A Computational Perspective", Springer, 2nd edition, 229-231
 
     """
     n = int(n)
@@ -606,10 +626,10 @@ def pollard_pm1(n, B=10, a=2, retries=0, seed=1234):
     References
     ==========
 
-    - Richard Crandall & Carl Pomerance (2005), "Prime Numbers:
-      A Computational Perspective", Springer, 2nd edition, 236-238
-    - http://modular.math.washington.edu/edu/2007/spring/ent/ent-html/node81.html
-    - http://www.cs.toronto.edu/~yuvalf/Factorization.pdf
+    .. [1] Richard Crandall & Carl Pomerance (2005), "Prime Numbers:
+           A Computational Perspective", Springer, 2nd edition, 236-238
+    .. [2] http://modular.math.washington.edu/edu/2007/spring/ent/ent-html/node81.html
+    .. [3] https://www.cs.toronto.edu/~yuvalf/Factorization.pdf
     """
 
     n = int(n)
@@ -967,8 +987,8 @@ def factorint(n, limit=None, use_trial=True, use_rho=True, use_pm1=True,
                                use_rho=use_rho, use_pm1=use_pm1,
                                verbose=verbose, visual=False)
     elif isinstance(n, Mul):
-        factordict = dict([(int(k), int(v)) for k, v in
-                           list(n.as_powers_dict().items())])
+        factordict = {int(k): int(v) for k, v in
+            n.as_powers_dict().items()}
     elif isinstance(n, dict):
         factordict = n
     if factordict and (isinstance(n, Mul) or isinstance(n, dict)):
@@ -1360,8 +1380,11 @@ def divisors(n, generator=False):
     >>> list(divisors(120, generator=True))
     [1, 2, 4, 8, 3, 6, 12, 24, 5, 10, 20, 40, 15, 30, 60, 120]
 
+    Notes
+    =====
+
     This is a slightly modified version of Tim Peters referenced at:
-    http://stackoverflow.com/questions/1010381/python-factorization
+    https://stackoverflow.com/questions/1010381/python-factorization
 
     See Also
     ========
@@ -1387,10 +1410,8 @@ def divisor_count(n, modulus=1):
     Return the number of divisors of ``n``. If ``modulus`` is not 1 then only
     those that are divisible by ``modulus`` are counted.
 
-    References
-    ==========
-
-    - http://www.mayer.dial.pipex.com/maths/formulae.htm
+    Examples
+    ========
 
     >>> from sympy import divisor_count
     >>> divisor_count(6)
@@ -1400,6 +1421,7 @@ def divisor_count(n, modulus=1):
     ========
 
     factorint, divisors, totient
+
     """
 
     if not modulus:
@@ -1436,12 +1458,6 @@ def udivisors(n, generator=False):
     prime factors. If only the number of unitary divisors is desired use
     udivisor_count(n).
 
-    References
-    ==========
-
-    - http://en.wikipedia.org/wiki/Unitary_divisor
-    - http://mathworld.wolfram.com/UnitaryDivisor.html
-
     Examples
     ========
 
@@ -1458,6 +1474,13 @@ def udivisors(n, generator=False):
     ========
 
     primefactors, factorint, divisors, divisor_count, udivisor_count
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Unitary_divisor
+    .. [2] http://mathworld.wolfram.com/UnitaryDivisor.html
+
     """
 
     n = as_int(abs(n))
@@ -1477,10 +1500,13 @@ def udivisor_count(n):
     """
     Return the number of unitary divisors of ``n``.
 
-    References
+    Parameters
     ==========
 
-    - http://mathworld.wolfram.com/UnitaryDivisorFunction.html
+    n : integer
+
+    Examples
+    ========
 
     >>> from sympy.ntheory.factor_ import udivisor_count
     >>> udivisor_count(120)
@@ -1490,6 +1516,12 @@ def udivisor_count(n):
     ========
 
     factorint, divisors, udivisors, divisor_count, totient
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/UnitaryDivisorFunction.html
+
     """
 
     if n == 0:
@@ -1519,11 +1551,6 @@ def antidivisors(n, generator=False):
     Antidivisors [1]_ of n are numbers that do not divide n by the largest
     possible margin.  If generator is True an unordered generator is returned.
 
-    References
-    ==========
-
-    .. [1] definition is described in http://oeis.org/A066272/a066272a.html
-
     Examples
     ========
 
@@ -1538,6 +1565,12 @@ def antidivisors(n, generator=False):
     ========
 
     primefactors, factorint, divisors, divisor_count, antidivisor_count
+
+    References
+    ==========
+
+    .. [1] definition is described in https://oeis.org/A066272/a066272a.html
+
     """
 
     n = as_int(abs(n))
@@ -1553,10 +1586,10 @@ def antidivisor_count(n):
     """
     Return the number of antidivisors [1]_ of ``n``.
 
-    References
+    Parameters
     ==========
 
-    .. [1] formula from https://oeis.org/A066272
+    n : integer
 
     Examples
     ========
@@ -1571,6 +1604,12 @@ def antidivisor_count(n):
     ========
 
     factorint, divisors, antidivisors, divisor_count, totient
+
+    References
+    ==========
+
+    .. [1] formula from https://oeis.org/A066272
+
     """
 
     n = as_int(abs(n))
@@ -1587,11 +1626,10 @@ class totient(Function):
     ``totient(n)`` or `\phi(n)` is the number of positive integers `\leq` n
     that are relatively prime to n.
 
-    References
+    Parameters
     ==========
 
-    .. [1] https://en.wikipedia.org/wiki/Euler%27s_totient_function
-    .. [2] http://mathworld.wolfram.com/TotientFunction.html
+    n : integer
 
     Examples
     ========
@@ -1606,6 +1644,13 @@ class totient(Function):
     ========
 
     divisor_count
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Euler%27s_totient_function
+    .. [2] http://mathworld.wolfram.com/TotientFunction.html
+
     """
     @classmethod
     def eval(cls, n):
@@ -1632,12 +1677,6 @@ class reduced_totient(Function):
     ``reduced_totient(n)`` or `\lambda(n)` is the smallest m > 0 such that
     `k^m \equiv 1 \mod n` for all k relatively prime to n.
 
-    References
-    ==========
-
-    .. [1] https://en.wikipedia.org/wiki/Carmichael_function
-    .. [2] http://mathworld.wolfram.com/CarmichaelFunction.html
-
     Examples
     ========
 
@@ -1653,6 +1692,13 @@ class reduced_totient(Function):
     ========
 
     totient
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Carmichael_function
+    .. [2] http://mathworld.wolfram.com/CarmichaelFunction.html
+
     """
     @classmethod
     def eval(cls, n):
@@ -1693,7 +1739,10 @@ class divisor_sigma(Function):
     Parameters
     ==========
 
-    k : power of divisors in the sum
+    n : integer
+
+    k : integer, optional
+        power of divisors in the sum
 
         for k = 0, 1:
         ``divisor_sigma(n, 0)`` is equal to ``divisor_count(n)``
@@ -1701,11 +1750,6 @@ class divisor_sigma(Function):
 
         Default for k is 1.
 
-    References
-    ==========
-
-    .. [1] http://en.wikipedia.org/wiki/Divisor_function
-
     Examples
     ========
 
@@ -1723,6 +1767,12 @@ class divisor_sigma(Function):
     ========
 
     divisor_count, totient, divisors, factorint
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Divisor_function
+
     """
 
     @classmethod
@@ -1758,18 +1808,16 @@ def core(n, t=2):
     Parameters
     ==========
 
-    t : core(n, t) calculates the t-th power free part of n
+    n : integer
+
+    t : integer
+        core(n, t) calculates the t-th power free part of n
 
         ``core(n, 2)`` is the squarefree part of ``n``
         ``core(n, 3)`` is the cubefree part of ``n``
 
         Default for t is 2.
 
-    References
-    ==========
-
-    .. [1] http://en.wikipedia.org/wiki/Square-free_integer#Squarefree_core
-
     Examples
     ========
 
@@ -1787,6 +1835,12 @@ def core(n, t=2):
     ========
 
     factorint, sympy.solvers.diophantine.square_factor
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Square-free_integer#Squarefree_core
+
     """
 
     n = as_int(n)
@@ -1863,11 +1917,6 @@ class udivisor_sigma(Function):
 
         Default for k is 1.
 
-    References
-    ==========
-
-    .. [1] http://mathworld.wolfram.com/UnitaryDivisorFunction.html
-
     Examples
     ========
 
@@ -1886,6 +1935,12 @@ class udivisor_sigma(Function):
 
     divisor_count, totient, divisors, udivisors, udivisor_count, divisor_sigma,
     factorint
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/UnitaryDivisorFunction.html
+
     """
 
     @classmethod
@@ -1915,11 +1970,6 @@ class primenu(Function):
     .. math ::
         \nu(n) = k.
 
-    References
-    ==========
-
-    .. [1] http://mathworld.wolfram.com/PrimeFactor.html
-
     Examples
     ========
 
@@ -1933,6 +1983,12 @@ class primenu(Function):
     ========
 
     factorint
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/PrimeFactor.html
+
     """
 
     @classmethod
@@ -1960,11 +2016,6 @@ class primeomega(Function):
     .. math ::
         \Omega(n) = \sum_{i=1}^k m_i.
 
-    References
-    ==========
-
-    .. [1] http://mathworld.wolfram.com/PrimeFactor.html
-
     Examples
     ========
 
@@ -1978,6 +2029,12 @@ class primeomega(Function):
     ========
 
     factorint
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/PrimeFactor.html
+
     """
 
     @classmethod
@@ -1988,3 +2045,183 @@ class primeomega(Function):
                 raise ValueError("n must be a positive integer")
             else:
                 return sum(factorint(n).values())
+
+
+def mersenne_prime_exponent(nth):
+    """Returns the exponent ``i`` for the nth Mersenne prime (which
+    has the form `2^i - 1`).
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory.factor_ import mersenne_prime_exponent
+    >>> mersenne_prime_exponent(1)
+    2
+    >>> mersenne_prime_exponent(20)
+    4423
+    """
+    n = as_int(nth)
+    if n < 1:
+        raise ValueError("nth must be a positive integer; mersenne_prime_exponent(1) == 2")
+    if n > 51:
+        raise ValueError("There are only 51 perfect numbers; nth must be less than or equal to 51")
+    return MERSENNE_PRIME_EXPONENTS[n - 1]
+
+
+def is_perfect(n):
+    """Returns True if ``n`` is a perfect number, else False.
+
+    A perfect number is equal to the sum of its positive, proper divisors.
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory.factor_ import is_perfect, divisors
+    >>> is_perfect(20)
+    False
+    >>> is_perfect(6)
+    True
+    >>> sum(divisors(6)[:-1])
+    6
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/PerfectNumber.html
+
+    """
+    from sympy.core.power import integer_log
+
+    r, b = integer_nthroot(1 + 8*n, 2)
+    if not b:
+        return False
+    n, x = divmod(1 + r, 4)
+    if x:
+        return False
+    e, b = integer_log(n, 2)
+    return b and (e + 1) in MERSENNE_PRIME_EXPONENTS
+
+
+def is_mersenne_prime(n):
+    """Returns True if  ``n`` is a Mersenne prime, else False.
+
+    A Mersenne prime is a prime number having the form `2^i - 1`.
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory.factor_ import is_mersenne_prime
+    >>> is_mersenne_prime(6)
+    False
+    >>> is_mersenne_prime(127)
+    True
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/MersennePrime.html
+
+    """
+    from sympy.core.power import integer_log
+
+    r, b = integer_log(n + 1, 2)
+    return b and r in MERSENNE_PRIME_EXPONENTS
+
+
+def abundance(n):
+    """Returns the difference between the sum of the positive
+    proper divisors of a number and the number.
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory import abundance, is_perfect, is_abundant
+    >>> abundance(6)
+    0
+    >>> is_perfect(6)
+    True
+    >>> abundance(10)
+    -2
+    >>> is_abundant(10)
+    False
+    """
+    return divisor_sigma(n, 1) - 2 * n
+
+
+def is_abundant(n):
+    """Returns True if ``n`` is an abundant number, else False.
+
+    A abundant number is smaller than the sum of its positive proper divisors.
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory.factor_ import is_abundant
+    >>> is_abundant(20)
+    True
+    >>> is_abundant(15)
+    False
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/AbundantNumber.html
+
+    """
+    n = as_int(n)
+    if is_perfect(n):
+        return False
+    return n % 6 == 0 or bool(abundance(n) > 0)
+
+
+def is_deficient(n):
+    """Returns True if ``n`` is a deficient number, else False.
+
+    A deficient number is greater than the sum of its positive proper divisors.
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory.factor_ import is_deficient
+    >>> is_deficient(20)
+    False
+    >>> is_deficient(15)
+    True
+
+    References
+    ==========
+
+    .. [1] http://mathworld.wolfram.com/DeficientNumber.html
+
+    """
+    n = as_int(n)
+    if is_perfect(n):
+        return False
+    return bool(abundance(n) < 0)
+
+
+def is_amicable(m, n):
+    """Returns True if the numbers `m` and `n` are "amicable", else False.
+
+    Amicable numbers are two different numbers so related that the sum
+    of the proper divisors of each is equal to that of the other.
+
+    Examples
+    ========
+
+    >>> from sympy.ntheory.factor_ import is_amicable, divisor_sigma
+    >>> is_amicable(220, 284)
+    True
+    >>> divisor_sigma(220) == divisor_sigma(284)
+    True
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Amicable_numbers
+
+    """
+    if m == n:
+        return False
+    a, b = map(lambda i: divisor_sigma(i), (m, n))
+    return a == b == (m + n)
