@@ -4,13 +4,12 @@ from sympy.plotting import (plot, plot_parametric, plot3d_parametric_line,
                             plot3d, plot3d_parametric_surface)
 from sympy.plotting.plot import unset_show, plot_contour
 from sympy.utilities import lambdify as lambdify_
-from sympy.utilities.pytest import skip, raises
+from sympy.utilities.pytest import skip, raises, warns
 from sympy.plotting.experimental_lambdify import lambdify
 from sympy.external import import_module
 
 from tempfile import NamedTemporaryFile
 import os
-import warnings
 
 unset_show()
 
@@ -279,15 +278,11 @@ def plot_and_save_4(name):
     # is the only way to evaluate the integral. We should perhaps just remove
     # that warning.
 
-    with warnings.catch_warnings(record=True) as w:
+    with warns(UserWarning, match="The evaluation of the expression is problematic"):
         i = Integral(log((sin(x)**2 + 1)*sqrt(x**2 + 1)), (x, 0, y))
         p = plot(i, (y, 1, 5))
         p.save(tmp_file('%s_advanced_integral' % name))
         p._backend.close()
-        # Make sure no other warnings were raised
-        for i in w:
-            assert issubclass(i.category, UserWarning)
-            assert "The evaluation of the expression is problematic" in str(i.message)
 
 def plot_and_save_5(name):
     tmp_file = TmpFileManager.tmp_file
@@ -434,3 +429,38 @@ def test_append_issue_7140():
 
     with raises(TypeError):
         p1.append(p2._series)
+
+def test_issue_15265():
+    from sympy.core.sympify import sympify
+    from sympy.core.singleton import S
+
+    matplotlib = import_module('matplotlib', min_module_version='1.1.0', catch=(RuntimeError,))
+    if not matplotlib:
+        skip("Matplotlib not the default backend")
+
+    x = Symbol('x')
+    eqn = sin(x)
+
+    p = plot(eqn, xlim=(-S.Pi, S.Pi), ylim=(-1, 1))
+    p._backend.close()
+
+    p = plot(eqn, xlim=(-1, 1), ylim=(-S.Pi, S.Pi))
+    p._backend.close()
+
+    p = plot(eqn, xlim=(-1, 1), ylim=(sympify('-3.14'), sympify('3.14')))
+    p._backend.close()
+
+    p = plot(eqn, xlim=(sympify('-3.14'), sympify('3.14')), ylim=(-1, 1))
+    p._backend.close()
+
+    raises(ValueError,
+        lambda: plot(eqn, xlim=(-S.ImaginaryUnit, 1), ylim=(-1, 1)))
+
+    raises(ValueError,
+        lambda: plot(eqn, xlim=(-1, 1), ylim=(-1, S.ImaginaryUnit)))
+
+    raises(ValueError,
+        lambda: plot(eqn, xlim=(-S.Infinity, 1), ylim=(-1, 1)))
+
+    raises(ValueError,
+        lambda: plot(eqn, xlim=(-1, 1), ylim=(-1, S.Infinity)))
