@@ -12,8 +12,7 @@ from sympy.core.symbol import symbols
 from sympy.functions import (sin, cos, exp, tan, cot, atan, asin, atanh,
     tanh, log, sqrt)
 from sympy.core.numbers import Rational
-from sympy.core import expand
-
+from sympy.core import expand, S
 
 def is_close(a, b):
     tol = 10**(-10)
@@ -153,7 +152,7 @@ def test_log():
     R, x = ring('x', QQ)
     p = 1 + x
     p1 = rs_log(p, x, 4)/x**2
-    assert p1 == 1/3*x - 1/2 + x**(-1)
+    assert p1 == S(1)/3*x - S(1)/2 + x**(-1)
     p = 1 + x +2*x**2/3
     p1 = rs_log(p, x, 9)
     assert p1 == -17*x**8/648 + 13*x**7/189 - 11*x**6/162 - x**5/45 + \
@@ -178,7 +177,7 @@ def test_log():
         EX(1/a)*x + EX(log(a))
 
     p = x + x**2 + 3
-    assert rs_log(p, x, 10).compose(x, 5) == EX(log(3) + 19281291595/9920232)
+    assert rs_log(p, x, 10).compose(x, 5) == EX(log(3) + S(19281291595)/9920232)
 
 def test_exp():
     R, x = ring('x', QQ)
@@ -283,7 +282,7 @@ def test_asin():
 def test_tan():
     R, x, y = ring('x, y', QQ)
     assert rs_tan(x, x, 9)/x**5 == \
-        17/315*x**2 + 2/15 + 1/3*x**(-2) + x**(-4)
+        S(17)/315*x**2 + S(2)/15 + S(1)/3*x**(-2) + x**(-4)
     assert rs_tan(x*y + x**2*y**3, x, 9) == 4*x**8*y**11/3 + 17*x**8*y**9/45 + \
         4*x**7*y**9/3 + 17*x**7*y**7/315 + x**6*y**9/3 + 2*x**6*y**7/3 + \
         x**5*y**7 + 2*x**5*y**5/15 + x**4*y**5 + x**3*y**3/3 + x**2*y**3 + x*y
@@ -291,11 +290,11 @@ def test_tan():
     # Constant term in series
     a = symbols('a')
     R, x, y = ring('x, y', QQ[tan(a), a])
-    assert rs_tan(x + a, x, 5) == (tan(a)**5 + 5*tan(a)**3/3 +
-        2*tan(a)/3)*x**4 + (tan(a)**4 + 4*tan(a)**2/3 + 1/3)*x**3 + \
+    assert rs_tan(x + a, x, 5) == (tan(a)**5 + 5*tan(a)**S(3)/3 +
+        2*tan(a)/3)*x**4 + (tan(a)**4 + 4*tan(a)**2/3 + S(1)/3)*x**3 + \
         (tan(a)**3 + tan(a))*x**2 + (tan(a)**2 + 1)*x + tan(a)
     assert rs_tan(x + x**2*y + a, x, 4) == (2*tan(a)**3 + 2*tan(a))*x**3*y + \
-        (tan(a)**4 + 4/3*tan(a)**2 + 1/3)*x**3 + (tan(a)**2 + 1)*x**2*y + \
+        (tan(a)**4 + S(4)/3*tan(a)**2 + S(1)/3)*x**3 + (tan(a)**2 + 1)*x**2*y + \
         (tan(a)**3 + tan(a))*x**2 + (tan(a)**2 + 1)*x + tan(a)
 
     R, x, y = ring('x, y', EX)
@@ -308,7 +307,7 @@ def test_tan():
         EX(tan(a)**2 + 1)*x + EX(tan(a))
 
     p = x + x**2 + 5
-    assert rs_atan(p, x, 10).compose(x, 10) == EX(atan(5) + 67701870330562640 / \
+    assert rs_atan(p, x, 10).compose(x, 10) == EX(atan(5) + S(67701870330562640) / \
         668083460499)
 
 def test_cot():
@@ -322,7 +321,7 @@ def test_cot():
 def test_sin():
     R, x, y = ring('x, y', QQ)
     assert rs_sin(x, x, 9)/x**5 == \
-        -1/5040*x**2 + 1/120 - 1/6*x**(-2) + x**(-4)
+        -S(1)/5040*x**2 + S(1)/120 - S(1)/6*x**(-2) + x**(-4)
     assert rs_sin(x*y + x**2*y**3, x, 9) == x**8*y**11/12 - \
         x**8*y**9/720 + x**7*y**9/12 - x**7*y**7/5040 - x**6*y**9/6 + \
         x**6*y**7/24 - x**5*y**7/2 + x**5*y**5/120 - x**4*y**5/2 - \
@@ -348,7 +347,7 @@ def test_sin():
 def test_cos():
     R, x, y = ring('x, y', QQ)
     assert rs_cos(x, x, 9)/x**5 == \
-        1/40320*x**3 - 1/720*x + 1/24*x**(-1) - 1/2*x**(-3) + x**(-5)
+        S(1)/40320*x**3 - S(1)/720*x + S(1)/24*x**(-1) - S(1)/2*x**(-3) + x**(-5)
     assert rs_cos(x*y + x**2*y**3, x, 9) == x**8*y**12/24 - \
         x**8*y**10/48 + x**8*y**8/40320 + x**7*y**10/6 - \
         x**7*y**8/120 + x**6*y**8/4 - x**6*y**6/720 + x**5*y**6/6 - \
@@ -382,7 +381,7 @@ def test_cos_sin():
 
 def test_atanh():
     R, x, y = ring('x, y', QQ)
-    assert rs_atanh(x, x, 9)/x**5 == 1/7*x**2 + 1/5 + 1/3*x**(-2) + x**(-4)
+    assert rs_atanh(x, x, 9)/x**5 == S(1)/7*x**2 + S(1)/5 + S(1)/3*x**(-2) + x**(-4)
     assert rs_atanh(x*y + x**2*y**3, x, 9) == 2*x**8*y**11 + x**8*y**9 + \
         2*x**7*y**9 + x**7*y**7/7 + x**6*y**9/3 + x**6*y**7 + x**5*y**7 + \
         x**5*y**5/5 + x**4*y**5 + x**3*y**3/3 + x**2*y**3 + x*y
@@ -400,12 +399,12 @@ def test_atanh():
         EX(1/(a**2 - 1))*x + EX(atanh(a))
 
     p = x + x**2 + 5
-    assert rs_atanh(p, x, 10).compose(x, 10) == EX(-733442653682135/5079158784 \
+    assert rs_atanh(p, x, 10).compose(x, 10) == EX(-S(733442653682135)/5079158784 \
         + atanh(5))
 
 def test_sinh():
     R, x, y = ring('x, y', QQ)
-    assert rs_sinh(x, x, 9)/x**5 == 1/5040*x**2 + 1/120 + 1/6*x**(-2) + x**(-4)
+    assert rs_sinh(x, x, 9)/x**5 == S(1)/5040*x**2 + S(1)/120 + S(1)/6*x**(-2) + x**(-4)
     assert rs_sinh(x*y + x**2*y**3, x, 9) == x**8*y**11/12 + \
         x**8*y**9/720 + x**7*y**9/12 + x**7*y**7/5040 + x**6*y**9/6 + \
         x**6*y**7/24 + x**5*y**7/2 + x**5*y**5/120 + x**4*y**5/2 + \
@@ -413,8 +412,8 @@ def test_sinh():
 
 def test_cosh():
     R, x, y = ring('x, y', QQ)
-    assert rs_cosh(x, x, 9)/x**5 == 1/40320*x**3 + 1/720*x + 1/24*x**(-1) + \
-        1/2*x**(-3) + x**(-5)
+    assert rs_cosh(x, x, 9)/x**5 == S(1)/40320*x**3 + S(1)/720*x + S(1)/24*x**(-1) + \
+        S(1)/2*x**(-3) + x**(-5)
     assert rs_cosh(x*y + x**2*y**3, x, 9) == x**8*y**12/24 + \
         x**8*y**10/48 + x**8*y**8/40320 + x**7*y**10/6 + \
         x**7*y**8/120 + x**6*y**8/4 + x**6*y**6/720 + x**5*y**6/6 + \
@@ -422,7 +421,7 @@ def test_cosh():
 
 def test_tanh():
     R, x, y = ring('x, y', QQ)
-    assert rs_tanh(x, x, 9)/x**5 == -17/315*x**2 + 2/15 - 1/3*x**(-2) + x**(-4)
+    assert rs_tanh(x, x, 9)/x**5 == -S(17)/315*x**2 + S(2)/15 - S(1)/3*x**(-2) + x**(-4)
     assert rs_tanh(x*y + x**2*y**3, x, 9) == 4*x**8*y**11/3 - \
         17*x**8*y**9/45 + 4*x**7*y**9/3 - 17*x**7*y**7/315 - x**6*y**9/3 + \
         2*x**6*y**7/3 - x**5*y**7 + 2*x**5*y**5/15 - x**4*y**5 - \
@@ -618,3 +617,14 @@ def test_rs_series():
 
     assert rs_series(sin(a)/7, a, 5).as_expr() == (sin(a)/7).series(a, 0,
             5).removeO()
+
+    assert rs_series(log(1 + x), x, 5).as_expr() == -x**4/4 + x**3/3 - \
+                    x**2/2 + x
+    assert rs_series(log(1 + 4*x), x, 5).as_expr() == -64*x**4 + 64*x**3/3 - \
+                    8*x**2 + 4*x
+    assert rs_series(log(1 + x + x**2), x, 10).as_expr() == -2*x**9/9 + \
+                    x**8/8 + x**7/7 - x**6/3 + x**5/5 + x**4/4 - 2*x**3/3 + \
+                    x**2/2 + x
+    assert rs_series(log(1 + x*a**2), x, 7).as_expr() == -x**6*a**12/6 + \
+                    x**5*a**10/5 - x**4*a**8/4 + x**3*a**6/3 - \
+                    x**2*a**4/2 + x*a**2
